'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _sighCore = require('sigh-core');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _sighCoreLibStream = require('sigh-core/lib/stream');

function eventCompiler() {
  var babel = require('babel');
  var _ = require('lodash');
  var instances = {};

  return function (instance, event) {
    if (instance.createInstance) {
      var newInstance = instance.instance;
      instances[newInstance] = instance.opts;
      return {};
    }
    var opts = instances[instance];

    var babelOpts = {
      modules: opts.modules,
      filename: event.path,
      sourceMap: true,
      moduleIds: true
    };

    if (opts.modules !== 'common') {
      var modulePath = event.projectPath.replace(/\.js$/, '');
      if (opts.getModulePath) modulePath = opts.getModulePath(modulePath);
      babelOpts.moduleId = modulePath;
    }

    // if (event.basePath)
    //   babelOpts.filenameRelative = event.basePath

    var result = babel.transform(event.data, babelOpts);
    return _.pick(result, 'code', 'map');
  };
}

// (de)serialise argument to and result of babel subprocess
function adaptEvent(instance, compiler) {
  return function (event) {
    if (event.type !== 'add' && event.type !== 'change') return event;

    var fileType = event.fileType;

    if (fileType !== 'js' && fileType !== 'es6') return event;

    var result = compiler(instance, _lodash2['default'].pick(event, 'type', 'data', 'path', 'projectPath'));

    // without proc pool a Promise.resolve is needed here
    return result.then(function (result) {
      event.data = result.code;
      event.applySourceMap(result.map);
      return event;
    });
  };
}

var pooledProc;
var lastInstance = 0;

exports['default'] = function (op, opts) {
  opts = _lodash2['default'].assign({ modules: 'amd' }, opts || {});

  if (!pooledProc) pooledProc = op.procPool.prepare(eventCompiler);

  ++lastInstance;
  pooledProc.all({ createInstance: true, instance: lastInstance, opts: opts });

  return (0, _sighCoreLibStream.mapEvents)(op.stream, adaptEvent(lastInstance, pooledProc));
};

module.exports = exports['default'];
//# sourceMappingURL=index.js.map